import tensorflow as tf
import requests


class SendProgress(tf.keras.callbacks.Callback):
    """
    This callback is passed to the training function of the model, to send the progress to the frontend used by SANEZOO
    """

    def __init__(self, dat_id, total_epochs):
        super().__init__()
        self.dataset_id = dat_id
        self.total_epochs = total_epochs

    def on_epoch_end(self, epoch, logs):
        """
        Sends the progress to the client (acting as a server) used by SANEZOO via a http request
        :param epoch: number
        :param logs: loss, validation loss
        """

        progress = {
            'state': 'DONE' if epoch+1 == self.total_epochs else 'RUN',
            'dataset_id': self.dataset_id,
            'progress': epoch*100/self.total_epochs,
            'metrics': {
                'train_loss': logs['loss'],
                'validation_loss': logs['val_loss'],
                'accuracy': logs['binary_accuracy']
            }
        }
        requests.post("http://localhost:9646/api/iteration/status", json=progress)
        print(epoch, logs)
        return super().on_epoch_end(epoch, logs)
