import numpy as np
import tensorflow as tf
from aiohttp import web
from sklearn.utils import class_weight


def patch_up(train, json_input, dimension):
    """
    Creates (square) patches of specified size from the (training) images, this is a data duplication technique
    :param train: images
    :param json_input: dictionary containing parameters (passed as JSON request to API)
    :param dimension: size of patches
    :return: patched images
    """

    patch_size = json_input['preprocessing_params']['patch_size']
    patch_overlap = json_input['preprocessing_params']['overlap'] / 100
    train = np.array(tf.image.extract_patches(
        images=train,
        sizes=[1, patch_size, patch_size, 1],
        strides=[1, patch_size * (1 - patch_overlap), patch_size * (1 - patch_overlap), 1],
        rates=[1, 1, 1, 1],
        padding='VALID')
    )
    train = train.reshape(-1, patch_size, patch_size, dimension)
    return train


def error_response(reason):
    """
    Error response of the API
    :param reason: for the error
    :return: JSON response
    """

    return web.json_response({
        'status': 'ERROR',
        'error_reason': reason,
        'error_code': 400
    }, status=400, )


def calc_class_weights(train_y):
    """
    Calculate weights of labels
    :param train_y: labeled data (masks)
    :return: weights for labels
    """
    weights = class_weight.compute_class_weight('balanced', classes=np.unique(train_y), y=train_y.flatten())
    return np.sqrt(np.sqrt(weights))
