import tensorflow as tf


def make_and_save_model(name):
    """
    Creates and saves an encoder-decoder
    :param name:
    :return:
    """

    input = tf.keras.layers.Input(shape=(None, None, 3))
    x = input

    skip_layers = []
    layer_number = 3
    dimension_number = 8

    # Encoder
    for i in range(layer_number):
        x = tf.keras.layers.Conv2D(dimension_number, (3, 3), activation="relu", padding="same")(x)
        skip_layers.append(x)
        x = tf.keras.layers.MaxPooling2D((2, 2), padding="same")(x)
        dimension_number *= 2

    # Middle
    dimension_number /= 4
    x = tf.keras.layers.Conv2D(dimension_number, (3, 3), activation="relu", padding="same")(x)
    x = tf.keras.layers.Conv2D(dimension_number, (3, 3), activation="relu", padding="same")(x)
    dimension_number *= 2

    # Decoder
    for i in range(layer_number):
        x = tf.keras.layers.Conv2DTranspose(dimension_number, (2, 2), strides=2, padding="same")(x)
        x = tf.keras.layers.Concatenate(axis=3)([skip_layers.pop(), x])
        x = tf.keras.layers.Conv2D(dimension_number, (3, 3), activation="relu", padding="same")(x)
        dimension_number /= 2

    # End
    x = tf.keras.layers.Conv2D(dimension_number, (3, 3), activation="relu", padding="same")(x)
    x = tf.keras.layers.Conv2D(1, kernel_size=1, activation="sigmoid", padding="same")(x)

    # Autoencoder
    autoencoder = tf.keras.models.Model(input, x)
    # print(autoencoder.summary())
    autoencoder.save(name)


# for manually creating models
if __name__ == '__main__':
    make_and_save_model('testing')
