import tensorflow as tf
from matplotlib import pyplot as plt

class PlotProgress(tf.keras.callbacks.Callback):
    loss = []
    val_loss = []
    bin_accuracy = []
    val_bin_accuracy = []

    def on_train_begin(self, logs=None):
        self.loss.clear()
        self.val_loss.clear()
        self.bin_accuracy.clear()
        self.val_bin_accuracy.clear()
        return super().on_train_begin(logs)

    def on_train_end(self, logs=None):
        plt.plot(range(len(self.loss)), self.loss, label='loss')
        plt.plot(range(len(self.val_loss)), self.val_loss, label='validation loss')
        # plt.plot(range(len(self.bin_accuracy)), self.bin_accuracy, label='binary accuracy')
        # plt.plot(range(len(self.val_bin_accuracy)), self.val_bin_accuracy, label='validation binary accuracy')
        plt.xlabel('epochs')
        plt.ylabel('losses')
        plt.title('loss decline over training')
        plt.legend()
        plt.savefig('graph_loss{}.png'.format(self.loss[-1]))
        plt.clf()
        plt.plot(range(len(self.bin_accuracy)), self.bin_accuracy, label='binary accuracy')
        plt.plot(range(len(self.val_bin_accuracy)), self.val_bin_accuracy, label='validation binary accuracy')
        plt.xlabel('epochs')
        plt.ylabel('accuracy')
        plt.title('binary accuracy over training')
        plt.legend()
        plt.savefig('graph_accuracy{}.png'.format(self.bin_accuracy[-1]))
        plt.clf()
        return super().on_train_end(logs)

    def on_epoch_end(self, epoch, logs=None):
        print('epoch:', epoch)
        print(logs)
        self.loss.append(logs['loss'])
        self.val_loss.append(logs['val_loss'])
        self.bin_accuracy.append(logs['binary_accuracy'])
        self.val_bin_accuracy.append(logs['val_binary_accuracy'])