import asyncio

import socketio
from aiohttp import web
import tensorflow as tf
from concurrent.futures import ThreadPoolExecutor

sio = socketio.AsyncServer(cors_allowed_origins='*')
progress_reporter = web.Application()
sio.attach(progress_reporter)
executor = ThreadPoolExecutor(max_workers=1)
# datasetID_client = dict()


@sio.event
async def connect(sid, env):
    print(sid, "CONNECTED - SYNC", env)
    # datasetID_client[env['HTTP_ID']] = sid


@sio.event
async def disconnect(sid):
    print(sid, 'disconnected')
    # datasetID_client.pop(list(datasetID_client.keys())[list(datasetID_client.values()).index(sid)])


async def send_progress(queue):
    """
    Asynchronously waits for a message in the queue, forwards the message to connected clients
    :param queue: used for communication with API
    """

    while True:
        message = await asyncio.get_event_loop().run_in_executor(executor, queue.get)
        # if message['dataset_id'] in datasetID_client:
        #     print('sending progress')
        await sio.emit('report', data=message)


class SendProgressThroughWebSocket(tf.keras.callbacks.Callback):
    """
    This callback is passed to the training function of the model, to pass the progress to the progress reporter used with demonstrative frontend
    """

    def __init__(self, dat_id, total_epochs, queue):
        super().__init__()
        self.dataset_id = dat_id
        self.total_epochs = total_epochs
        self.queue = queue

    def on_epoch_end(self, epoch, logs=None):
        """
        Sends the progress to the progress reporter via a multiprocessing queue
        :param epoch: number
        :param logs: loss, validation loss
        """

        self.queue.put({'state': 'DONE' if epoch+1 == self.total_epochs else 'RUN', 'dataset_id': self.dataset_id, 'progress': (epoch+1)*100/self.total_epochs})
        # print(epoch, logs)
        return super().on_epoch_end(epoch, logs)
